package jp.co.atmarkit.user.web.logic {
import flash.events.Event;
import flash.events.MouseEvent;

import jp.co.atmarkit.user.cnst.MessageConsts;
import jp.co.atmarkit.user.dto.UserRequestDto;
import jp.co.atmarkit.user.dto.UserResponseDto;
import jp.co.atmarkit.user.entity.User;
import jp.co.atmarkit.user.logic.Logic;
import jp.co.atmarkit.user.web.UserForm;

import mx.collections.ArrayCollection;
import mx.collections.IViewCursor;
import mx.controls.Alert;
import mx.controls.advancedDataGridClasses.AdvancedDataGridColumn;
import mx.events.FlexEvent;
import mx.events.ListEvent;
import mx.rpc.events.FaultEvent;
import mx.rpc.events.ResultEvent;
    
/**
 * クイズアプリケーションのロジッククラス
 */
public class UserLogic extends Logic {
//--------------------------------------
//  Variables
//--------------------------------------

    [Bindable]
    /** ユーザーのリスト */
    public var userList:ArrayCollection;

    [Bindable]
    /** ヒット件数 */
    public var hitNum:String;

    /** 選択された性別絞り込み条件 */
    private var selectedSex:String;

    /** 選択された血液型絞り込み条件 */
    private var selectedBloodType:String;

    /** 選択された都道府県絞り込み条件 */
    private var selectedPre:String;

    /** 入力されたEメール絞り込み条件 */
    private var selectedEmail:String;
    
//--------------------------------------
//  Constructor
//--------------------------------------
    /**
     * コンストラクタ
     */
    public function UserLogic() {
    }

//--------------------------------------
//  Initialization
//--------------------------------------
    /**
     * onCreationCompleteHandler
     * 
     * @param event FlexEvent
     */
    override protected function onCreationCompleteHandler(event:FlexEvent):void {
    }

//--------------------------------------
//  UI Event Handler
//--------------------------------------
    /**
     * getButton押下処理
     * 
     * @param event MouseEvent
     */
    public function getButtonClickHandler(event:MouseEvent):void {
        // DTOの準備
        var requestDto:UserRequestDto = new UserRequestDto();
        
        // 特に送信するデータが無いので空のまま処理を呼び出し
        
        // リモートメソッドを呼び出す
        view.srvS2BlazeDS.getUser(requestDto);
        
        // 利用不可処理
        view.controlBar.enabled = false;
    }

//--------------------------------------
//  Server Connection Event Handler
//--------------------------------------
    
    /** 
     * ユーザーをサーバーから取得する：結果イベント
     * 
     * @param event ResultEvent
     */
    public function getUserResultHandler(event:ResultEvent):void {
        // DTOを取得
        var responseDto:UserResponseDto = event.result as UserResponseDto;
        
        // 結果を取得
        userList = responseDto.userList;
        
        // 共通後処理
        initDisplay();
    }

    /** 
     * ユーザーをサーバーから取得する：失敗イベント
     * 
     * @param event FaultEvent
     */
    public function getUserFaultHandler(event:FaultEvent):void {
        trace(event.fault.message);
        Alert.show(MessageConsts.ERROR_SERVER_ERROR);
        
        // 共通後処理
        initDisplay();
    }
    
    /**
     * サーバー処理後の共通後処理
     */
    private function initDisplay():void {
        // 利用可処理
        view.controlBar.enabled = true;
        
        // フィルター条件を初期状態にする
        view.sexCombo.selectedIndex = 0;
        view.bloodCombo.selectedIndex = 0;
        view.preCombo.selectedIndex = 0;
        view.emailText.text = "";
        
        // 表示件数更新
        updateHitNum();
    }

//--------------------------------------
//  Functions
//--------------------------------------
    
    /**
     * 行番号ラベルファンクション
     * 
     * @param object 選択行のデータ
     * @param column AdvancedDataGridColumn
     * @return 行番号
     */
    public function rowNumLabelFunction(object:Object, column:AdvancedDataGridColumn):String {
        var rowIndex:int = userList.getItemIndex(object) + 1;
        return String(rowIndex);
    }
  
    /**
     * 誕生日ラベルファンクション
     * 
     * @param object 選択行のデータ
     * @param column AdvancedDataGridColumn
     * @return 誕生日
     */
    public function birthdayLabelFunction(object:Object, column:AdvancedDataGridColumn):String {
        var strings:Array = User(object).birthday.split("/");
        
        if(String(strings[1]).length == 1) {
            strings[1] = "0" + strings[1]
        }
        
        if(String(strings[2]).length == 1) {
            strings[2] = "0" + strings[2]
        }
        
        return strings[0] + "/" + strings[1] + "/" + strings[2];
    }
  
    /**
     * 性別フィルターファンクション
     * 
     * @param event ListEvent
     */
    public function sexChangeHandler(event:ListEvent):void {
        if(userList != null) {
            if(view.sexCombo.selectedItem.value == "全性別") {
                selectedSex = "";
            } else {
                selectedSex = view.sexCombo.selectedItem.value;
            }
            
            // フィルターファンクション
            userList.filterFunction = doFilter;
            
            // フィルター処理を実行
            userList.refresh();

            // 表示件数更新
            updateHitNum();
        }
    }

    /**
     * 血液型フィルターファンクション
     * 
     * @param event ListEvent
     */
    public function bloodChangeHandler(event:ListEvent):void {
        if(userList != null) {
            if(view.bloodCombo.selectedItem.value == "全血液型") {
                selectedBloodType = "";
            } else {
                selectedBloodType = view.bloodCombo.selectedItem.value;
            }
            
            // フィルターファンクション
            userList.filterFunction = doFilter;
            
            // フィルター処理を実行
            userList.refresh();

            // 表示件数更新
            updateHitNum();
        }
    }

    /**
     * 都道府県フィルターファンクション
     * 
     * @param event ListEvent
     */
    public function preChangeHandler(event:ListEvent):void {
        if(userList != null) {
            if(view.preCombo.selectedItem.value == "全都道府県") {
                selectedPre = "";
            } else {
                selectedPre = view.preCombo.selectedItem.value;
            }
            
            // フィルターファンクション
            userList.filterFunction = doFilter;
            
            // フィルター処理を実行
            userList.refresh();

            // 表示件数更新
            updateHitNum();
        }
    }
    
    /**
     * Eメールフィルターファンクション
     * 
     * @param event Event
     */
    public function emailChangeHandler(event:Event):void {
        if(userList != null) {
            if(view.emailText.text == "") {
                selectedEmail = "";
            } else {
                selectedEmail = view.emailText.text;
            }
            
            // フィルターファンクション
            userList.filterFunction = doFilter;
            
            // フィルター処理を実行
            userList.refresh();
            
            // 表示件数更新
            updateHitNum();
        }
    }

    /**
     * メインフィルター処理
     * 
     * @param フィルター対象のオブジェクト1行1行
     * @return 行がフィルター条件に合うか合わないか
     */
    private function doFilter(value:Object):Boolean {
        var sexCheck:Boolean;
        var bloodTypeCheck:Boolean;
        var preCheck:Boolean;
        var emailCheck:Boolean;
        
        if(selectedSex == "" || selectedSex == null) {
            sexCheck = true;
        } else {
            sexCheck = (User(value).sex == selectedSex);
        }

        if(selectedBloodType == "" || selectedBloodType == null) {
            bloodTypeCheck = true;
        } else {
            bloodTypeCheck = (User(value).bloodtype == selectedBloodType);
        }

        if(selectedPre == "" || selectedPre == null) {
            preCheck = true;
        } else {
            preCheck = (User(value).prefecture == selectedPre);
        }
        
        if(selectedEmail == "" || selectedEmail == null) {
            emailCheck = true;
        } else {
            emailCheck = (User(value).mail.indexOf(selectedEmail) == 0);
        }

        return (sexCheck && bloodTypeCheck && preCheck && emailCheck);
    }
    
    /**
     * 表示件数更新
     */
    private function updateHitNum():void {
        if(userList != null) {
            hitNum = "（" + userList.length + "/" + userList.source.length + "）";
        }
    }
        
//--------------------------------------
//  View-Logic Binding
//--------------------------------------
    /** 画面 */
    public var _view:UserForm;
    
    /**
     * 画面を取得します
     * 
     * @return 画面
     */     
    public function get view():UserForm {
        if (_view == null) {
            _view = super.document as UserForm;
        }
        return _view;
    }
    
    /**
     * 画面をセットします。
     * 
     * @param view セットする画面
     */
    public function set view(view:UserForm):void {
        _view = view;
    }
}
}